/**********************************************************************
* Simple application for converting U-Boot environment settings 
* on the pcDuino from and editable binary format to U-Boot binary
*
* Enjoy!
* Bill
***********************************************************************
* Copyright (c) 2013 by William Greathouse, Brecksville, OH, USA
*
* Permission is granted to use the source code within this
* file in whole or in part for any use, personal or commercial,
* without restriction or limitation.
*
* No warranties, either explicit or implied, are made as to the
* suitability of this code for any purpose. Use at your own risk.
**********************************************************************/

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include "crc32.h"

static struct envfile
{
    uint32_t crc32;
    char env[];
} *envbuf;

static int envsize = 128*1024;
static int verbose = 1;
static int cursize = 0;

static void syntax(void)
{
    fprintf(stderr, "Syntax: envgen filename\n\n");
    fprintf(stderr, "Read the U-Boot environment text file and output\n"
                    "the binary formatted environment with checksum.\n"
                    "The resulting binary can be written to NAND flash\n");
}

int main(int argc, char *argv[])
{
    FILE *envf;
    char *ptr;
    char linebuf[1024];


    if (argc < 2)
    {
        syntax();
        return 1;
    }

    envbuf = calloc(1, envsize);
    if (envbuf == NULL)
    {
        perror("Out of memory\n");
        return 3;
    }

    envf = fopen(argv[1], "rt");
    if (envf == NULL)
    {
        perror(argv[1]);
        syntax();
        return 4;
    }

    /* Loop on file input adding to environment */
    cursize = sizeof(uint32_t);
    ptr = &envbuf->env[0];
    while (!feof(envf))
    {
        if (fgets(linebuf, sizeof(linebuf), envf))
        {
            int l=strlen(linebuf)-1;
            if (l > 0)
            {
                linebuf[strlen(linebuf)-1]='\0';
                if (verbose)
                    fprintf(stderr, "%s\n", linebuf);
                cursize += (l+1);
                if (cursize < envsize)
                {
                    memcpy(ptr, linebuf, l+1); /* include string terminator */
                    ptr += (l+1);
                }    
                else
                    fprintf(stderr, "Environment too large, truncated\n");
            }
        }
    }

    /* Create CRC */
    envbuf->crc32 = crc32 (0, (uint8_t *)(envbuf->env), envsize-sizeof(uint32_t));

    if (verbose)
        fprintf(stderr, "CRC32: %08x\n", envbuf->crc32);

    fwrite(envbuf, envsize, 1, stdout);

    fclose(envf);

    return 0;
}

